
# q-expansion of newform 1080.2.f.d, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1080
weight = 2
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[271, [1, 0]], [541, [1, 0]], [1001, [1, 0]], [217, [-1, 0]]]
aps_data = [[0, 0], [0, 0], [2, -1], [0, 2], [2, 0], [0, 2], [0, 3], [1, 0], [0, 5], [10, 0], [-9, 0], [0, -8], [2, 0], [0, 6], [0, 0], [0, 1], [6, 0], [-7, 0], [0, 8], [14, 0], [0, -14], [-1, 0], [0, -5], [-6, 0], [0, -8], [0, 0], [0, 16], [0, -4], [15, 0], [0, 2], [0, 0], [0, 0], [0, 17], [-4, 0], [-6, 0], [-20, 0], [0, 14], [0, -20], [0, -21], [0, -9], [4, 0], [-23, 0], [-24, 0], [0, -14], [0, -3], [8, 0], [-23, 0], [0, -12], [0, -23], [-15, 0], [0, -30], [-20, 0], [-1, 0], [-20, 0], [0, -3], [0, -24], [-24, 0], [11, 0], [0, 30], [18, 0], [0, 18], [0, 23], [0, -20], [-6, 0], [0, 4], [0, 27], [-20, 0], [0, -10], [0, -8], [-5, 0], [0, -30], [20, 0], [0, -14], [0, 20], [-5, 0], [0, 21], [6, 0], [0, 30], [4, 0], [19, 0], [30, 0], [-17, 0], [34, 0], [0, 6], [-15, 0], [0, -9], [18, 0], [0, -36], [24, 0], [0, -40], [0, -27], [-26, 0], [0, 2], [20, 0], [13, 0], [0, 25], [8, 0], [10, 0], [0, 2], [-26, 0], [0, 2], [0, 18], [0, 36], [6, 0], [7, 0], [0, 38], [0, 23], [0, -29], [2, 0], [-3, 0], [0, -14], [0, 32], [0, 19], [20, 0], [5, 0], [0, 0], [0, -30], [0, -1], [0, 33], [6, 0], [38, 0], [0, -38], [0, 6], [0, -25], [21, 0], [28, 0], [-10, 0], [32, 0], [0, 12], [0, -24], [-9, 0], [0, 36], [-19, 0], [0, -20], [-38, 0], [-25, 0], [0, -7], [0, -50], [0, -39], [-18, 0], [48, 0], [-36, 0], [0, -10], [0, -9], [-22, 0], [12, 0], [0, -28], [0, 7], [-37, 0], [0, -3], [0, -22], [-38, 0], [0, 46], [0, -57], [0, -8], [30, 0], [-48, 0], [-50, 0], [0, 20], [-4, 0], [0, -27], [0, 6], [0, 44], [-24, 0], [0, 38], [0, -7], [49, 0], [0, -6], [37, 0], [0, 39], [40, 0], [27, 0], [-54, 0], [0, 32], [31, 0], [14, 0], [-33, 0], [2, 0], [0, -2], [6, 0], [0, -2], [12, 0], [0, 24], [0, 6], [0, -33], [-16, 0], [0, -22], [0, -4], [-49, 0], [-30, 0], [0, 16], [0, -3], [61, 0], [0, 0], [0, -15], [0, -53], [1, 0], [0, -52], [0, 2], [0, -16], [-18, 0], [40, 0], [0, -16], [-31, 0], [-46, 0], [0, -34], [-16, 0], [0, 43], [16, 0], [4, 0], [0, -26], [30, 0], [0, 56], [0, 52], [-32, 0], [13, 0], [0, -22], [-64, 0], [0, 31], [0, 61], [38, 0], [5, 0], [6, 0], [0, -6], [0, 1], [25, 0], [0, 15], [34, 0], [0, 32], [24, 0], [0, -10], [45, 0], [-48, 0], [68, 0], [0, -38], [0, -15], [-29, 0], [0, 27], [64, 0], [54, 0], [0, 52], [-13, 0], [0, 16], [43, 0], [0, -6], [20, 0], [0, -68], [-42, 0], [-36, 0], [0, -13], [0, 12], [48, 0], [0, -3], [-1, 0], [0, 29], [0, 0], [-7, 0], [0, 62], [0, 43], [0, -6], [0, -46], [0, 47], [5, 0], [0, 16], [0, 21], [60, 0], [64, 0], [68, 0], [0, 36], [0, 78], [43, 0], [0, -40], [0, 26], [53, 0], [0, 78], [0, -70], [0, 35], [-57, 0], [-39, 0], [50, 0], [0, -32], [-41, 0], [0, 7], [-5, 0], [0, -32], [-6, 0], [0, 4], [0, -6], [-73, 0], [-16, 0], [16, 0], [0, 31], [0, -62], [60, 0], [0, 6], [-36, 0], [-37, 0], [0, 42], [-56, 0], [0, -50], [0, 34], [0, -61], [16, 0]]
